import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;
import java.util.List;
import java.util.Scanner;

public class App {
    // #region app constants
    private static final Scanner scanner = new Scanner(System.in);
    private static Teacher[] teachers = new Teacher[Teacher.nameMap.size()]; // problem

    // Room channel numbers, sorted in ascending order: 37, 38, 39
    private static final int[] ROOM_NUMBERS = { 1521263, 1364580, 1521262 };

    // Date constants for Start and End Dates
    private static final String[] DATES = { "2024-11-04", "2024-11-05", "2024-11-06", "2024-11-07", "2024-11-08" };

    // #region Initialization
    public static URL generateLink(int channelNumber, String date) throws MalformedURLException {
        String baseUrl = "https://api.thingspeak.com/channels/";
        String formattedStartDateTime = date + " 07:00:00";
        String formattedEndDateTime = date + " 19:00:00";

        return new URL(baseUrl + channelNumber + "/feeds.csv?start=" +
                formattedStartDateTime.replace(" ", "%20") +
                "&end=" + formattedEndDateTime.replace(" ", "%20"));
    }

    private static void calculateBreakPoints(Break calcBreak, Teacher teacher, Teacher nextTeacher) {
        int duration = calcBreak.getEnd() - calcBreak.getStart();
        Co2Data[] minuteData = calcBreak.getCo2Datas();

        if (minuteData.length != duration) {
            System.out.println("Data inconsistency detected: duration mismatch.");
            return;
        }

        int breakPoints = minuteData.length;
        for (int i = 0; i < duration - 1; i++) {
            if (minuteData[i].getCo2Level() < minuteData[i + 1].getCo2Level()) {
                breakPoints--;
            }
        }

        if (duration > 5) {
            teacher.addPoints(0, breakPoints, 0);
        } else {
            teacher.addPoints(breakPoints, 0, 0);
        }

        if (teacher.getShortName() != nextTeacher.getShortName()) {
            // bonus Point for teacher switch
        }
    }

    private static void initializeTeachers() {
        // Mapping short names to full teacher names
        Teacher.nameMap.put("Hm", "Markus Hofer");
        Teacher.nameMap.put("Bd", "Larissa Binder");
        Teacher.nameMap.put("Bu", "Dominic Baur");
        Teacher.nameMap.put("Cg", "Stefan Castelberg");
        Teacher.nameMap.put("Do", "Corinna Dohr");
        Teacher.nameMap.put("Eh", "Matthias Ehrensperger");
        Teacher.nameMap.put("Fh", "Lukas Fehr");
        Teacher.nameMap.put("Gi", "Johannes Giesinger ");
        Teacher.nameMap.put("Gr", "Fabian Gross");
        Teacher.nameMap.put("Hi", "Anne-Kathrin Hiestand");
        Teacher.nameMap.put("Kg", "Ulrich Kugler");
        Teacher.nameMap.put("Kh", "Salome Kühne");
        Teacher.nameMap.put("Lz", "Valentino Leanza");
        Teacher.nameMap.put("Lu", "Manuela Lucke");
        Teacher.nameMap.put("Or", "Esther Ortolani");
        Teacher.nameMap.put("Re", "Nadja Reuteler");
        Teacher.nameMap.put("Se", "Christian Steiner");
        Teacher.nameMap.put("Ts", "Georg Tscholl");
        Teacher.nameMap.put("Vt", "Martin Vogt");
        Teacher.nameMap.put("Zu", "Andrina Zumbühl ");
        Teacher.nameMap.put("Kp", "Yannick Kampl");
        Teacher.nameMap.put("Ba", "Pierre Barès");
        Teacher.nameMap.put("Bz", "Pascal Benz");
        Teacher.nameMap.put("Hn", "Mario Hermann");
        Teacher.nameMap.put("Le", "Regula Lehmann");
        Teacher.nameMap.put("Du", "Myriam Dudli");
        Teacher.nameMap.put("Bt", "Armin Baumgartner");
        Teacher.nameMap.put("Es", "Roger Eugster");
        Teacher.nameMap.put("Hu", "Astrid Hubov");
        Teacher.nameMap.put("Ge", "Pascal Griffel");

        // Initialize the teachers array with the size of nameMap
        teachers = new Teacher[Teacher.nameMap.size()];

        int index = 0;
        for (String initial : Teacher.nameMap.keySet()) {
            Teacher teacher = new Teacher(initial);
            teachers[index++] = teacher;
        }
    }

    // #region User Interaction
    private static int getUserInput(String textOutput) {
        System.out.println(textOutput);
        while (true) {
            if (scanner.hasNextInt()) {
                return scanner.nextInt();
            }
            System.out.println("Invalid input. Please enter a number.");
            scanner.next();
        }
    }

    private static void interactWithUser() {
        while (true) {
            int userInput = getUserInput("Do you want to see how the points were calculated? (1 = Yes, 0 = No)");
            if (userInput == 1) {
                printExplanation();
                break;
            } else if (userInput == 0) {
                break;
            } else {
                System.out.println("Invalid input. Please enter 1 for Yes or 0 for No.");
            }
        }
        printShutdown();
    }

    // #region Explanation
    private static void printExplanation() {
        System.out.println("Point calculation explanation:");
        System.out.println("1. Up to 5 points for keeping the window open during a small pause.");
        System.out.println("2. Up to 15-20 points for long pauses, depending on window usage.");
        System.out.println("3. 5 bonus points for teacher switches in classrooms.");
    }

    // #region Shutdown
    private static void printShutdown() {
        System.out.println("Shutting down...");
        for (int i = 3; i > 0; i--) {
            System.out.print(i + "...");
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

    // #region Print & Sort
    private static void sortTeachers() {
        Arrays.sort(teachers,
                (a, b) -> Integer.compare(b.getTotalPoints(), a.getTotalPoints()));
    }

    private static void printTeachers() {
        int rank = 1;
        int previousPoints = -1;

        for (int i = 0; i < teachers.length; i++) {
            Teacher teacher = teachers[i];
            int teacherPoints = teacher.getTotalPoints();

            if (teacherPoints != previousPoints) {
                rank = i + 1;
                previousPoints = teacherPoints;
            }
            System.out.println(rank + ". " + teacher.getName() + " - Points: " + teacherPoints);
        }
    }

    private static Teacher findTeacherForBreak(int breakIndex, TimeTable timeTable, int dayIndex) {
        String teacherShortName = timeTable.getLesson(breakIndex, dayIndex);

        if (teacherShortName == null || teacherShortName.isEmpty()) {
            System.out.println("No teacher assigned during this break.");
            return null;
        }
        if (!Teacher.nameMap.containsKey(teacherShortName) && teacherShortName != "Lunch") {
            System.out.println("Teacher with short name " + teacherShortName + " not found in Teacher.nameMap.");
            return null;
        }

        String fullName = Teacher.nameMap.get(teacherShortName);
        for (Teacher teacher : teachers) {
            if (teacher.getName().equals(fullName)) {
                return teacher;
            }
        }
        return null;
    }

    // #region main
    public static void main(String[] args) {
        try {
            System.out.println("Calculations in process. Please do not shut off...");
            initializeTeachers();

            for (int roomIndex = 0; roomIndex < ROOM_NUMBERS.length; roomIndex++) {
                TimeTable timeTable = new TimeTable(roomIndex);

                for (String date : DATES) {
                    try {
                        List<Co2Data> tempData = Co2Data.getData(generateLink(ROOM_NUMBERS[roomIndex], date));
                        if (tempData == null || tempData.isEmpty()) {
                            System.out
                                    .println("No data available for " + date + " in Room" + (roomIndex + 37) + ".");
                            continue;
                        }

                        Break[] shortBreaks = BreakSchedule.createBreaks(
                                BreakSchedule.START_SMALL_BREAK, BreakSchedule.END_SMALL_BREAK, tempData);

                        Break[] longBreaks = BreakSchedule.createBreaks(
                                BreakSchedule.START_LONG_BREAK, BreakSchedule.END_LONG_BREAK, tempData);

                        for (int i = 0; i < shortBreaks.length; i++) {
                            Teacher teacher = findTeacherForBreak(i, timeTable, Arrays.asList(DATES).indexOf(date));
                            Teacher nextTeacher = findTeacherForBreak(i + 1, timeTable,
                                    Arrays.asList(DATES).indexOf(date));
                            if (teacher != null) {
                                if (nextTeacher == null) {
                                    nextTeacher = teacher;
                                }
                                calculateBreakPoints(shortBreaks[i], teacher, nextTeacher);
                            }
                        }

                        for (int i = 0; i < longBreaks.length; i++) {
                            Teacher teacher = findTeacherForBreak(i, timeTable, Arrays.asList(DATES).indexOf(date));
                            Teacher nextTeacher = findTeacherForBreak(i + 1, timeTable,
                                    Arrays.asList(DATES).indexOf(date));
                            if (teacher != null) {
                                if (nextTeacher == null) {
                                    nextTeacher = teacher;
                                }
                                calculateBreakPoints(longBreaks[i], teacher, nextTeacher);
                            }
                        }
                    } catch (Exception e) {
                        System.out.println("Error processing data for date: " + date + ". " + e.getMessage());
                    }
                }
            }

            sortTeachers();
            printTeachers();
            interactWithUser();

        } catch (Exception e) {
            System.out.println("Fatal error: " + e.getMessage());
        } finally {
            scanner.close();
        }
    }
}
